@@ -1,6 +1,7 @@
 from DAQmxTypes import TaskHandle
 import DAQmxFunctions
 from DAQmxFunctions import *
+import ctypes
 
 
 # Create a list of the name of the function that uses TastHandle as the first argument
@@ -22,7 +23,8 @@ except NotImplementedError:
     _callback = False
 
 if _callback:
-    class CallbackParent():
+    class CallbackParent(object):
+        _EveryNSamplesEvent_already_register = False
         def AutoRegisterEveryNSamplesEvent(self, everyNsamplesEventType,nSamples,options, name='EveryNCallback'):
             """Register the method named name as the callback function for EveryNSamplesEvent
             
@@ -31,8 +33,12 @@ if _callback:
             The parameters everyNsamplesEventType, nSamples and options are the same 
             as the DAQmxRegisterEveryNSamplesEvent parameters
             
-            No parameters are passed to the method        
+            No parameters are passed to the method  
+
+            If an event was already registered, the UnregisterEveryNSamplesEvent is automatically called      
             """
+            if self._EveryNSamplesEvent_already_register:
+                self.UnregisterEveryNSamplesEvent()
             self_id = create_callbackdata_id(self)
             # Define the python function
             def EveryNCallback_py(taskHandle, everyNsamplesEventType, nSamples, self_id):
@@ -43,6 +49,11 @@ if _callback:
             self.EveryNCallback_C = DAQmxEveryNSamplesEventCallbackPtr(EveryNCallback_py)
             # Register the function
             self.RegisterEveryNSamplesEvent(everyNsamplesEventType,nSamples,options,self.EveryNCallback_C,self_id)
+            self._EveryNSamplesEvent_already_register = True
+        def UnregisterEveryNSamplesEvent(self):
+            self.RegisterEveryNSamplesEvent(1,0,0,ctypes.cast(None, DAQmxEveryNSamplesEventCallbackPtr),0)
+            self._EveryNSamplesEvent_already_register = False
+
         def AutoRegisterDoneEvent(self, options, name='DoneCallback'):
             """Register the method named name as the callback function for DoneEvent
             
@@ -60,6 +71,7 @@ if _callback:
             self.DoneCallback_C = DAQmxDoneEventCallbackPtr(DoneCallback_py)
             # Register the function
             self.RegisterDoneEvent(options,self.DoneCallback_C,self_id)
+
         def AutoRegisterSignalEvent(self, signalID, options, name='SignalCallback'):
             """Register the method named name as the callback function for RegisterSignalEvent
             
@@ -80,8 +92,10 @@ if _callback:
             self.SignalCallback_C = DAQmxSignalEventCallbackPtr(SignalCallback_py)
             # Register the function
             self.RegisterSignalEvent(signalID, options, self.SignalCallback_C, self_id)
+
 else:
-    class CallbackParent():
+
+    class CallbackParent(object):
         def __getattr__(self, name):
             if name in ['AutoRegisterEveryNSamplesEvent', 'AutoRegisterDoneEvent', 'AutoRegisterSignalEvent']:
                 raise NotImplementedError, 'Callback methods are not available'
@@ -89,9 +103,16 @@ else:
 
 
 class Task(CallbackParent):
-    def __init__(self):
+    def __init__(self, name=""):
         self.taskHandle = TaskHandle(0)
-        DAQmxCreateTask(b"",byref(self.taskHandle))
+        DAQmxCreateTask(name,byref(self.taskHandle))
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *exc_info):
+        self.ClearTask()
+
     def __del__(self):
         """ Clear automatically the task to be able to reallocate resources """ 
         # Clear the task before deleting the object
@@ -102,7 +123,7 @@ class Task(CallbackParent):
         # b has the same taskHandle as a, and deleting a will clear the task of b   
         try: 
             self.ClearTask()
-        except DAQError:
+        except Exception:
             pass
     def ClearTask(self):
         if self.taskHandle:
@@ -123,7 +144,7 @@ class Task(CallbackParent):
         doc = 'T.%s(%s) -> error.' %(name, ', '.join(arg_names[1:]))
         cmd = """def {0}(self, {1}):
         "{3}"
-        {2}(self.taskHandle, {1})"""
+        return {2}(self.taskHandle, {1})"""
         exec(cmd.format(name, ', '.join(arg_names[1:]), function_name, doc))    
     del function_name, name, func, arg_names, doc
 
